@@ -1,4 +1,4 @@
-# frozen_string_literal: false
+# frozen_string_literal: true
 #
 #   loader.rb -
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
@@ -98,13 +98,13 @@ module IRB # :nodoc:
 
     def old # :nodoc:
       back_io = @io
-      back_path = @irb_path
+      back_path = irb_path
       back_name = @irb_name
       back_scanner = @irb.scanner
       begin
         @io = FileInputMethod.new(path)
         @irb_name = File.basename(path)
-        @irb_path = path
+        self.irb_path = path
         @irb.signal_status(:IN_LOAD) do
           if back_io.kind_of?(FileInputMethod)
             @irb.eval_input
@@ -119,7 +119,7 @@ module IRB # :nodoc:
       ensure
         @io = back_io
         @irb_name = back_name
-        @irb_path = back_path
+        self.irb_path = back_path
         @irb.scanner = back_scanner
       end
     end
