@@ -40,27 +40,10 @@ class Array # :nodoc:
 end
 
 ##
-# \Module \MakeMakefile is used by Ruby C extensions to generate a Makefile which will
+# mkmf.rb is used by Ruby C extensions to generate a Makefile which will
 # correctly compile and link the C extension to Ruby and a third-party
 # library.
 module MakeMakefile
-
-  target_rbconfig = nil
-  ARGV.delete_if do |arg|
-    opt = arg.delete_prefix("--target-rbconfig=")
-    unless opt == arg
-      target_rbconfig = opt
-    end
-  end
-  if target_rbconfig
-    # Load the RbConfig for the target platform into this module.
-    # Cross-compiling needs the same version of Ruby.
-    Kernel.load target_rbconfig, self
-  else
-    # The RbConfig for the target platform where the built extension runs.
-    RbConfig = ::RbConfig
-  end
-
   #### defer until this module become global-state free.
   # def self.extended(obj)
   #   obj.init_mkmf
@@ -76,9 +59,6 @@ module MakeMakefile
   # The makefile configuration using the defaults from when Ruby was built.
 
   CONFIG = RbConfig::MAKEFILE_CONFIG
-
-  ##
-  # The saved original value of +LIB+ environment variable
   ORIG_LIBPATH = ENV['LIB']
 
   ##
@@ -265,16 +245,12 @@ MESSAGE
   CSRCFLAG = CONFIG['CSRCFLAG']
   CPPOUTFILE = config_string('CPPOUTFILE') {|str| str.sub(/\bconftest\b/, CONFTEST)}
 
-  # :startdoc:
-
-  # Removes _files_.
   def rm_f(*files)
     opt = (Hash === files.last ? [files.pop] : [])
     FileUtils.rm_f(Dir[*files.flatten], *opt)
   end
   module_function :rm_f
 
-  # Removes _files_ recursively.
   def rm_rf(*files)
     opt = (Hash === files.last ? [files.pop] : [])
     FileUtils.rm_rf(Dir[*files.flatten], *opt)
@@ -289,8 +265,6 @@ MESSAGE
     t if times.all? {|n| n <= t}
   end
 
-  # :stopdoc:
-
   def split_libs(*strs)
     sep = $mswin ? /\s+/ : /\s+(?=-|\z)/
     strs.flat_map {|s| s.lstrip.split(sep)}
@@ -416,11 +390,6 @@ MESSAGE
       env, *commands = commands if Hash === commands.first
       envs.merge!(env) if env
     end
-
-    # disable ASAN leak reporting - conftest programs almost always don't bother
-    # to free their memory.
-    envs['LSAN_OPTIONS'] = "detect_leaks=0" unless ENV.key?('LSAN_OPTIONS')
-
     return envs, expand[commands]
   end
 
@@ -428,23 +397,15 @@ MESSAGE
     envs.map {|e, v| "#{e}=#{v.quote}"}
   end
 
-  # :startdoc:
-
-  # call-seq:
-  #   xsystem(command, werror: false)   -> true or false
-  #
-  # Executes _command_ with expanding variables, and returns the exit
-  # status like as Kernel#system.  If _werror_ is true and the error
-  # output is not empty, returns +false+.  The output will logged.
-  def xsystem(command, werror: false)
+  def xsystem command, opts = nil
     env, command = expand_command(command)
     Logging::open do
       puts [env_quote(env), command.quote].join(' ')
-      if werror
+      if opts and opts[:werror]
         result = nil
         Logging.postpone do |log|
           output = IO.popen(env, command, &:read)
-          result = $?.success?
+          result = ($?.success? and File.zero?(log.path))
           output
         end
         result
@@ -454,7 +415,6 @@ MESSAGE
     end
   end
 
-  # Executes _command_ similarly to xsystem, but yields opened pipe.
   def xpopen command, *mode, &block
     env, commands = expand_command(command)
     command = [env_quote(env), command].join(' ')
@@ -469,7 +429,6 @@ MESSAGE
     end
   end
 
-  # Logs _src_
   def log_src(src, heading="checked program was")
     src = src.split(/^/)
     fmt = "%#{src.size.to_s.size}d: %s"
@@ -484,15 +443,10 @@ EOM
 EOM
   end
 
-  # Returns the language-dependent source file name for configuration
-  # checks.
   def conftest_source
     CONFTEST_C
   end
 
-  # Creats temporary source file from +COMMON_HEADERS+ and _src_.
-  # Yields the created source string and uses the returned string as
-  # the source code, if the block is given.
   def create_tmpsrc(src)
     src = "#{COMMON_HEADERS}\n#{src}"
     src = yield(src) if block_given?
@@ -513,8 +467,6 @@ EOM
     src
   end
 
-  # :stopdoc:
-
   def have_devel?
     unless defined? $have_devel
       $have_devel = true
@@ -523,7 +475,7 @@ EOM
     $have_devel
   end
 
-  def try_do(src, command, **opts, &b)
+  def try_do(src, command, *opts, &b)
     unless have_devel?
       raise <<MSG
 The compiler failed to generate an executable file.
@@ -532,7 +484,7 @@ MSG
     end
     begin
       src = create_tmpsrc(src, &b)
-      xsystem(command, **opts)
+      xsystem(command, *opts)
     ensure
       log_src(src)
     end
@@ -583,42 +535,42 @@ MSG
   end
 
   def libpathflag(libpath=$DEFLIBPATH|$LIBPATH)
-    libpathflags = nil
     libpath.map{|x|
       case x
       when "$(topdir)", /\A\./
         LIBPATHFLAG
       else
-        libpathflags ||= [LIBPATHFLAG, RPATHFLAG].grep(/\S/).join(" ")
+        LIBPATHFLAG+RPATHFLAG
       end % x.quote
-    }.join(" ")
-  end
-
-  def werror_flag(opt = nil)
-    config_string("WERRORFLAG") {|flag| opt = opt && !opt.empty? ? "#{opt} #{flag}" : flag}
-    opt
+    }.join
   end
 
   def with_werror(opt, opts = nil)
-    opt = werror_flag(opt) if opts and (opts = opts.dup).delete(:werror)
-    yield(opt, opts)
+    if opts
+      if opts[:werror] and config_string("WERRORFLAG") {|flag| opt = opt ? "#{opt} #{flag}" : flag}
+        (opts = opts.dup).delete(:werror)
+      end
+      yield(opt, opts)
+    else
+      yield(opt)
+    end
   end
 
-  def try_link0(src, opt = "", ldflags: "", **opts, &b) # :nodoc:
+  def try_link0(src, opt="", *opts, &b) # :nodoc:
     exe = CONFTEST+$EXEEXT
-    cmd = link_command(ldflags, opt)
+    cmd = link_command("", opt)
     if $universal
       require 'tmpdir'
       Dir.mktmpdir("mkmf_", oldtmpdir = ENV["TMPDIR"]) do |tmpdir|
         begin
           ENV["TMPDIR"] = tmpdir
-          try_do(src, cmd, **opts, &b)
+          try_do(src, cmd, *opts, &b)
         ensure
           ENV["TMPDIR"] = oldtmpdir
         end
       end
     else
-      try_do(src, cmd, **opts, &b)
+      try_do(src, cmd, *opts, &b)
     end and File.executable?(exe) or return nil
     exe
   ensure
@@ -627,32 +579,31 @@ MSG
 
   # Returns whether or not the +src+ can be compiled as a C source and linked
   # with its depending libraries successfully.  +opt+ is passed to the linker
-  # as options. Note that <tt>$CFLAGS</tt> and <tt>$LDFLAGS</tt> are also
-  # passed to the linker.
+  # as options. Note that +$CFLAGS+ and +$LDFLAGS+ are also passed to the
+  # linker.
   #
   # If a block given, it is called with the source before compilation. You can
   # modify the source in the block.
   #
   # [+src+] a String which contains a C source
   # [+opt+] a String which contains linker options
-  def try_link(src, opt = "", **opts, &b)
-    exe = try_link0(src, opt, **opts, &b) or return false
+  def try_link(src, opt="", *opts, &b)
+    exe = try_link0(src, opt, *opts, &b) or return false
     MakeMakefile.rm_f exe
     true
   end
 
   # Returns whether or not the +src+ can be compiled as a C source.  +opt+ is
-  # passed to the C compiler as options. Note that <tt>$CFLAGS</tt> is also
-  # passed to the compiler.
+  # passed to the C compiler as options. Note that +$CFLAGS+ is also passed to
+  # the compiler.
   #
   # If a block given, it is called with the source before compilation. You can
   # modify the source in the block.
   #
   # [+src+] a String which contains a C source
   # [+opt+] a String which contains compiler options
-  def try_compile(src, opt = "", werror: nil, **opts, &b)
-    opt = werror_flag(opt) if werror
-    try_do(src, cc_command(opt), werror: werror, **opts, &b) and
+  def try_compile(src, opt="", *opts, &b)
+    with_werror(opt, *opts) {|_opt, *| try_do(src, cc_command(_opt), *opts, &b)} and
       File.file?("#{CONFTEST}.#{$OBJEXT}")
   ensure
     MakeMakefile.rm_f "#{CONFTEST}*"
@@ -660,15 +611,15 @@ MSG
 
   # Returns whether or not the +src+ can be preprocessed with the C
   # preprocessor.  +opt+ is passed to the preprocessor as options. Note that
-  # <tt>$CFLAGS</tt> is also passed to the preprocessor.
+  # +$CFLAGS+ is also passed to the preprocessor.
   #
   # If a block given, it is called with the source before preprocessing. You
   # can modify the source in the block.
   #
   # [+src+] a String which contains a C source
   # [+opt+] a String which contains preprocessor options
-  def try_cpp(src, opt = "", **opts, &b)
-    try_do(src, cpp_command(CPPOUTFILE, opt), **opts, &b) and
+  def try_cpp(src, opt="", *opts, &b)
+    try_do(src, cpp_command(CPPOUTFILE, opt), *opts, &b) and
       File.file?("#{CONFTEST}.i")
   ensure
     MakeMakefile.rm_f "#{CONFTEST}*"
@@ -685,14 +636,6 @@ MSG
     end
   end
 
-  # :startdoc:
-
-  # Sets <tt>$CPPFLAGS</tt> to _flags_ and yields.  If the block returns a
-  # falsy value, <tt>$CPPFLAGS</tt> is reset to its previous value, remains
-  # set to _flags_ otherwise.
-  #
-  # [+flags+] a C preprocessor flag as a +String+
-  #
   def with_cppflags(flags)
     cppflags = $CPPFLAGS
     $CPPFLAGS = flags.dup
@@ -701,29 +644,20 @@ MSG
     $CPPFLAGS = cppflags unless ret
   end
 
-  # :nodoc:
-  def try_cppflags(flags, werror: true, **opts)
-    try_header(MAIN_DOES_NOTHING, flags, werror: werror, **opts)
+  def try_cppflags(flags, opts = {})
+    try_header(MAIN_DOES_NOTHING, flags, {:werror => true}.update(opts))
   end
 
-  # Check whether each given C preprocessor flag is acceptable and append it
-  # to <tt>$CPPFLAGS</tt> if so.
-  #
-  # [+flags+] a C preprocessor flag as a +String+ or an +Array+ of them
-  #
-  def append_cppflags(flags, **opts)
+  def append_cppflags(flags, *opts)
     Array(flags).each do |flag|
       if checking_for("whether #{flag} is accepted as CPPFLAGS") {
-           try_cppflags(flag, **opts)
+           try_cppflags(flag, *opts)
          }
         $CPPFLAGS << " " << flag
       end
     end
   end
 
-  # Sets <tt>$CFLAGS</tt> to _flags_ and yields.  If the block returns a falsy
-  # value, <tt>$CFLAGS</tt> is reset to its previous value, remains set to
-  # _flags_ otherwise.
   def with_cflags(flags)
     cflags = $CFLAGS
     $CFLAGS = flags.dup
@@ -732,14 +666,10 @@ MSG
     $CFLAGS = cflags unless ret
   end
 
-  # :nodoc:
-  def try_cflags(flags, werror: true, **opts)
-    try_compile(MAIN_DOES_NOTHING, flags, werror: werror, **opts)
+  def try_cflags(flags, opts = {})
+    try_compile(MAIN_DOES_NOTHING, flags, {:werror => true}.update(opts))
   end
 
-  # Sets <tt>$LDFLAGS</tt> to _flags_ and yields.  If the block returns a
-  # falsy value, <tt>$LDFLAGS</tt> is reset to its previous value, remains set
-  # to _flags_ otherwise.
   def with_ldflags(flags)
     ldflags = $LDFLAGS
     $LDFLAGS = flags.dup
@@ -748,30 +678,21 @@ MSG
     $LDFLAGS = ldflags unless ret
   end
 
-  # :nodoc:
-  def try_ldflags(flags, werror: $mswin, **opts)
-    try_link(MAIN_DOES_NOTHING, "", ldflags: flags, werror: werror, **opts)
+  def try_ldflags(flags, opts = {})
+    opts = {:werror => true}.update(opts) if $mswin
+    try_link(MAIN_DOES_NOTHING, flags, opts)
   end
 
-  # :startdoc:
-
-  # Check whether each given linker flag is acceptable and append it to
-  # <tt>$LDFLAGS</tt> if so.
-  #
-  # [+flags+] a linker flag as a +String+ or an +Array+ of them
-  #
-  def append_ldflags(flags, **opts)
+  def append_ldflags(flags, *opts)
     Array(flags).each do |flag|
       if checking_for("whether #{flag} is accepted as LDFLAGS") {
-           try_ldflags(flag, **opts)
+           try_ldflags(flag, *opts)
          }
         $LDFLAGS << " " << flag
       end
     end
   end
 
-  # :stopdoc:
-
   def try_static_assert(expr, headers = nil, opt = "", &b)
     headers = cpp_include(headers)
     try_compile(<<SRC, opt, &b)
@@ -860,7 +781,7 @@ int main() {printf("%"PRI_CONFTEST_PREFIX"#{neg ? 'd' : 'u'}\\n", conftest_const
         v
       }
       unless strvars.empty?
-        prepare << "char " << strvars.map {|v| %[#{v}[1024] = ""]}.join(", ") << "; "
+        prepare << "char " << strvars.map {|v| "#{v}[1024]"}.join(", ") << "; "
       end
     when nil
       call = ""
@@ -907,8 +828,6 @@ int t(void) { const volatile void *volatile p; p = &(&#{var})[0]; return !p; }
 SRC
   end
 
-  # :startdoc:
-
   # Returns whether or not the +src+ can be preprocessed with the C
   # preprocessor and matches with +pat+.
   #
@@ -947,8 +866,6 @@ SRC
     log_src(src)
   end
 
-  # :stopdoc:
-
   # This is used internally by the have_macro? method.
   def macro_defined?(macro, src, opt = "", &b)
     src = src.sub(/[^\n]\z/, "\\&\n")
@@ -968,8 +885,8 @@ SRC
   # * the linked file can be invoked as an executable
   # * and the executable exits successfully
   #
-  # +opt+ is passed to the linker as options. Note that <tt>$CFLAGS</tt> and
-  # <tt>$LDFLAGS</tt> are also passed to the linker.
+  # +opt+ is passed to the linker as options. Note that +$CFLAGS+ and
+  # +$LDFLAGS+ are also passed to the linker.
   #
   # If a block given, it is called with the source before compilation. You can
   # modify the source in the block.
@@ -1041,10 +958,6 @@ SRC
     format(LIBARG, lib) + " " + libs
   end
 
-  # Prints messages to $stdout, if verbose mode.
-  #
-  # Internal use only.
-  #
   def message(*s)
     unless Logging.quiet and not $VERBOSE
       printf(*s)
@@ -1076,10 +989,6 @@ SRC
     r
   end
 
-  # Build a message for checking.
-  #
-  # Internal use only.
-  #
   def checking_message(target, place = nil, opt = nil)
     [["in", place], ["with", opt]].inject("#{target}") do |msg, (pre, noun)|
       if noun
@@ -1104,10 +1013,10 @@ SRC
   #
   # [+flags+] a C compiler flag as a +String+ or an +Array+ of them
   #
-  def append_cflags(flags, **opts)
+  def append_cflags(flags, *opts)
     Array(flags).each do |flag|
       if checking_for("whether #{flag} is accepted as CFLAGS") {
-           try_cflags(flag, **opts)
+           try_cflags(flag, *opts)
          }
         $CFLAGS << " " << flag
       end
@@ -1341,7 +1250,6 @@ SRC
     end
   end
 
-  # :nodoc:
   # Returns whether or not the static type +type+ is defined.
   #
   # See also +have_type+
@@ -1399,7 +1307,6 @@ SRC
     end
   end
 
-  # :nodoc:
   # Returns whether or not the constant +const+ is defined.
   #
   # See also +have_const+
@@ -1559,7 +1466,7 @@ SRC
           u = "unsigned " if signed > 0
           prelude << "extern rbcv_typedef_ foo();"
           compat = UNIVERSAL_INTS.find {|t|
-            try_compile([prelude, "extern #{u}#{t} foo();"].join("\n"), opts, werror: true, &b)
+            try_compile([prelude, "extern #{u}#{t} foo();"].join("\n"), opts, :werror=>true, &b)
           }
         end
         if compat
@@ -1583,7 +1490,7 @@ SRC
   # Used internally by the what_type? method to determine if +type+ is a scalar
   # pointer.
   def scalar_ptr_type?(type, member = nil, headers = nil, &b)
-    try_compile(<<"SRC", &b)
+    try_compile(<<"SRC", &b)   # pointer
 #{cpp_include(headers)}
 /*top*/
 volatile #{type} conftestval;
@@ -1596,7 +1503,7 @@ SRC
   # Used internally by the what_type? method to determine if +type+ is a scalar
   # pointer.
   def scalar_type?(type, member = nil, headers = nil, &b)
-    try_compile(<<"SRC", &b)
+    try_compile(<<"SRC", &b)   # pointer
 #{cpp_include(headers)}
 /*top*/
 volatile #{type} conftestval;
@@ -1618,10 +1525,6 @@ SRC
     end
   end
 
-  # :startdoc:
-
-  # Returns a string represents the type of _type_, or _member_ of
-  # _type_ if _member_ is not +nil+.
   def what_type?(type, member = nil, headers = nil, &b)
     m = "#{type}"
     var = val = "*rbcv_var_"
@@ -1681,8 +1584,6 @@ SRC
     end
   end
 
-  # :nodoc:
-  #
   # This method is used internally by the find_executable method.
   #
   # Internal use only.
@@ -1721,6 +1622,8 @@ SRC
     nil
   end
 
+  # :startdoc:
+
   # Searches for the executable +bin+ on +path+.  The default path is your
   # +PATH+ environment variable. If that isn't defined, it will resort to
   # searching /usr/local/bin, /usr/ucb, /usr/bin and /bin.
@@ -1968,7 +1871,7 @@ SRC
       if pkgconfig = with_config("#{pkg}-config") and find_executable0(pkgconfig)
       # if and only if package specific config command is given
       elsif ($PKGCONFIG ||=
-             (pkgconfig = with_config("pkg-config") {config_string("PKG_CONFIG") || ENV["PKG_CONFIG"] || "pkg-config"}) &&
+             (pkgconfig = with_config("pkg-config") {config_string("PKG_CONFIG") || "pkg-config"}) &&
              find_executable0(pkgconfig) && pkgconfig) and
            xsystem([*envs, $PKGCONFIG, "--exists", pkg])
         # default to pkg-config command
@@ -1984,20 +1887,7 @@ SRC
           opts = Array(opts).map { |o| "--#{o}" }
           opts = xpopen([*envs, pkgconfig, *opts, *args], err:[:child, :out], &:read)
           Logging.open {puts opts.each_line.map{|s|"=> #{s.inspect}"}}
-          if $?.success?
-            opts = opts.strip
-            libarg, libpath = LIBARG, LIBPATHFLAG.strip
-            opts = opts.shellsplit.map { |s|
-              if s.start_with?('-l')
-                libarg % s[2..]
-              elsif s.start_with?('-L')
-                libpath % s[2..]
-              else
-                s
-              end
-            }.quote.join(" ")
-            opts
-          end
+          opts.strip if $?.success?
         }
       end
       orig_ldflags = $LDFLAGS
@@ -2179,10 +2069,9 @@ ARCH_FLAG = #{$ARCH_FLAG}
 DLDFLAGS = $(ldflags) $(dldflags) $(ARCH_FLAG)
 LDSHARED = #{CONFIG['LDSHARED']}
 LDSHAREDXX = #{config_string('LDSHAREDXX') || '$(LDSHARED)'}
-POSTLINK = #{config_string('POSTLINK', RbConfig::CONFIG)}
 AR = #{CONFIG['AR']}
-LD = #{CONFIG['LD']}
 EXEEXT = #{CONFIG['EXEEXT']}
+POSTLINK = #{CONFIG['POSTLINK']}
 
 }
     CONFIG.each do |key, val|
@@ -2299,6 +2188,7 @@ RULES
       line.gsub!(/\.o\b/, ".#{$OBJEXT}")
       line.gsub!(/\{\$\(VPATH\)\}/, "") unless $nmake
       line.gsub!(/\$\((?:hdr|top)dir\)\/config.h/, $config_h)
+      line.gsub!(%r"\$\(hdrdir\)/(?!ruby(?![^:;/\s]))(?=[-\w]+\.h)", '\&ruby/')
       if $nmake && /\A\s*\$\(RM|COPY\)/ =~ line
         line.gsub!(%r"[-\w\./]{2,}"){$&.tr("/", "\\")}
         line.gsub!(/(\$\((?!RM|COPY)[^:)]+)(?=\))/, '\1:/=\\')
@@ -2381,19 +2271,6 @@ RULES
   # directory, i.e. the current directory.  It is included as part of the
   # +VPATH+ and added to the list of +INCFLAGS+.
   #
-  # Yields the configuration part of the makefile to be generated, as an array
-  # of strings, if the block is given.  The returned value will be used the
-  # new configuration part.
-  #
-  #   create_makefile('foo') {|conf|
-  #     [
-  #       *conf,
-  #       "MACRO_YOU_NEED = something",
-  #     ]
-  #   }
-  #
-  # If "depend" file exist in the source directory, that content will be
-  # included in the generated makefile, with formatted by depend_rules method.
   def create_makefile(target, srcprefix = nil)
     $target = target
     libpath = $DEFLIBPATH|$LIBPATH
@@ -2502,7 +2379,7 @@ TARGET_ENTRY = #{EXPORT_PREFIX || ''}Init_$(TARGET_NAME)
 DLLIB = #{dllib}
 EXTSTATIC = #{$static || ""}
 STATIC_LIB = #{staticlib unless $static.nil?}
-#{!$extout && defined?($installed_list) ? %[INSTALLED_LIST = #{$installed_list}\n] : ""}
+#{!$extout && defined?($installed_list) ? "INSTALLED_LIST = #{$installed_list}\n" : ""}
 TIMESTAMP_DIR = #{$extout && $extmk ? '$(extout)/.timestamp' : '.'}
 " #"
     # TODO: fixme
@@ -2510,19 +2387,16 @@ TIMESTAMP_DIR = #{$extout && $extmk ? '$(extout)/.timestamp' : '.'}
     sodir = $extout ? '$(TARGET_SO_DIR)' : '$(RUBYARCHDIR)'
     n = '$(TARGET_SO_DIR)$(TARGET)'
     cleanobjs = ["$(OBJS)"]
-    cleanlibs = []
     if $extmk
       %w[bc i s].each {|ex| cleanobjs << "$(OBJS:.#{$OBJEXT}=.#{ex})"}
     end
     if target
       config_string('cleanobjs') {|t| cleanobjs << t.gsub(/\$\*/, "$(TARGET)#{deffile ? '-$(arch)': ''}")}
-      cleanlibs << '$(TARGET_SO)'
     end
-    config_string('cleanlibs') {|t| cleanlibs << t.gsub(/\$\*/) {n}}
     conf << "\
 TARGET_SO_DIR =#{$extout ? " $(RUBYARCHDIR)/" : ''}
 TARGET_SO     = $(TARGET_SO_DIR)$(DLLIB)
-CLEANLIBS     = #{cleanlibs.join(' ')}
+CLEANLIBS     = #{'$(TARGET_SO) ' if target}#{config_string('cleanlibs') {|t| t.gsub(/\$\*/) {n}}}
 CLEANOBJS     = #{cleanobjs.join(' ')} *.bak
 TARGET_SO_DIR_TIMESTAMP = #{timestamp_file(sodir, target_prefix)}
 " #"
@@ -2532,7 +2406,7 @@ TARGET_SO_DIR_TIMESTAMP = #{timestamp_file(sodir, target_prefix)}
     mfile.puts(conf)
     mfile.print "
 all:    #{$extout ? "install" : target ? "$(DLLIB)" : "Makefile"}
-static: #{$extmk && !$static ? "all" : %[$(STATIC_LIB)#{$extout ? " install-rb" : ""}]}
+static: #{$extmk && !$static ? "all" : "$(STATIC_LIB)#{$extout ? " install-rb" : ""}"}
 .PHONY: all install static install-so install-rb
 .PHONY: clean clean-so clean-static clean-rb
 " #"
@@ -2563,7 +2437,6 @@ static: #{$extmk && !$static ? "all" : %[$(STATIC_LIB)#{$extout ? " install-rb"
         mfile.puts dest
         mfile.print "clean-so::\n"
         mfile.print "\t-$(Q)$(RM) #{fseprepl[dest]} #{fseprepl[stamp]}\n"
-        mfile.print "\t-$(Q)$(RM_RF) #{fseprepl['$(CLEANLIBS)']}\n"
         mfile.print "\t-$(Q)$(RMDIRS) #{fseprepl[dir]}#{$ignore_error}\n"
       else
         mfile.print "#{f} #{stamp}\n"
@@ -2594,7 +2467,7 @@ static: #{$extmk && !$static ? "all" : %[$(STATIC_LIB)#{$extout ? " install-rb"
           dest = "#{dir}/#{File.basename(f)}"
           mfile.print("do-install-rb#{sfx}: #{dest}\n")
           mfile.print("#{dest}: #{f} #{timestamp_file(dir, target_prefix)}\n")
-          mfile.print("\t$(Q) $(#{$extout ? 'COPY' : 'INSTALL_DATA'}) #{f} $@\n")
+          mfile.print("\t$(Q) $(#{$extout ? 'COPY' : 'INSTALL_DATA'}) #{f} $(@D)\n")
           if defined?($installed_list) and !$extout
             mfile.print("\t@echo #{dest}>>$(INSTALLED_LIST)\n")
           end
@@ -2856,9 +2729,6 @@ MESSAGE
 
   split = Shellwords.method(:shellwords).to_proc
 
-  ##
-  # The prefix added to exported symbols automatically
-
   EXPORT_PREFIX = config_string('EXPORT_PREFIX') {|s| s.strip}
 
   hdr = ['#include "ruby.h"' "\n"]
@@ -2888,10 +2758,6 @@ MESSAGE
   # make compile rules
 
   COMPILE_RULES = config_string('COMPILE_RULES', &split) || %w[.%s.%s:]
-
-  ##
-  # Substitution in rules for NMake
-
   RULE_SUBST = config_string('RULE_SUBST')
 
   ##
@@ -2936,11 +2802,7 @@ MESSAGE
   ##
   # Argument which will add a library path to the linker
 
-  LIBPATHFLAG = config_string('LIBPATHFLAG') || '-L%s'
-
-  ##
-  # Argument which will add a runtime library path to the linker
-
+  LIBPATHFLAG = config_string('LIBPATHFLAG') || ' -L%s'
   RPATHFLAG = config_string('RPATHFLAG') || ''
 
   ##
@@ -2952,10 +2814,6 @@ MESSAGE
   # A C main function which does no work
 
   MAIN_DOES_NOTHING = config_string('MAIN_DOES_NOTHING') || "int main(int argc, char **argv)\n{\n  return !!argv[argc];\n}"
-
-  ##
-  # The type names for convertible_int
-
   UNIVERSAL_INTS = config_string('UNIVERSAL_INTS') {|s| Shellwords.shellwords(s)} ||
     %w[int short long long\ long]
 
@@ -2986,32 +2844,18 @@ realclean: distclean
 
   @lang = Hash.new(self)
 
-  ##
-  # Retrieves the module for _name_ language.
   def self.[](name)
     @lang.fetch(name)
   end
 
-  ##
-  # Defines the module for _name_ language.
   def self.[]=(name, mod)
     @lang[name] = mod
   end
 
-  ##
-  # The language that this module is for
-  LANGUAGE = -"C"
-
-  self[self::LANGUAGE] = self
-
-  cxx = Module.new do
-    # Module for C++
-
+  self["C++"] = Module.new do
     include MakeMakefile
     extend self
 
-    # :stopdoc:
-
     CONFTEST_CXX = "#{CONFTEST}.#{config_string('CXX_EXT') || CXX_EXT[0]}"
 
     TRY_LINK_CXX = config_string('TRY_LINK_CXX') ||
@@ -3041,12 +2885,7 @@ realclean: distclean
       conf = link_config(ldflags, *opts)
       RbConfig::expand(TRY_LINK_CXX.dup, conf)
     end
-
-    # :startdoc:
   end
-
-  cxx::LANGUAGE = -"C++"
-  self[cxx::LANGUAGE] = cxx
 end
 
 # MakeMakefile::Global = #
