@@ -7,25 +7,28 @@ require_relative "installer"
 require_relative "spec_fetcher"
 require_relative "user_interaction"
 require_relative "available_set"
+require_relative "deprecate"
 
 ##
 # Installs a gem along with all its dependencies from local and remote gems.
 
 class Gem::DependencyInstaller
   include Gem::UserInteraction
+  extend Gem::Deprecate
 
   DEFAULT_OPTIONS = { # :nodoc:
     env_shebang: false,
     document: %w[ri],
     domain: :both, # HACK: dup
     force: false,
-    format_executable: true, # HACK: dup
+    format_executable: false, # HACK: dup
     ignore_dependencies: false,
     prerelease: false,
     security_policy: nil, # HACK: NoSecurity requires OpenSSL. AlmostNo? Low?
     wrappers: true,
     build_args: nil,
     build_docs_in_background: false,
+    install_as_default: false,
   }.freeze
 
   ##
@@ -56,6 +59,7 @@ class Gem::DependencyInstaller
   # :format_executable:: See Gem::Installer#initialize.
   # :ignore_dependencies:: Don't install any dependencies.
   # :install_dir:: See Gem::Installer#install.
+  # :install_root:: See Gem::Installer#install.
   # :prerelease:: Allow prerelease versions.  See #install.
   # :security_policy:: See Gem::Installer::new and Gem::Security.
   # :user_install:: See Gem::Installer.new
@@ -83,8 +87,8 @@ class Gem::DependencyInstaller
     @user_install        = options[:user_install]
     @wrappers            = options[:wrappers]
     @build_args          = options[:build_args]
-    @build_jobs          = options[:build_jobs]
     @build_docs_in_background = options[:build_docs_in_background]
+    @install_as_default = options[:install_as_default]
     @dir_mode = options[:dir_mode]
     @data_mode = options[:data_mode]
     @prog_mode = options[:prog_mode]
@@ -97,7 +101,12 @@ class Gem::DependencyInstaller
     @installed_gems = []
     @toplevel_specs = nil
 
-    @cache_dir = options[:cache_dir] || @install_dir
+    @install_root = options[:install_root] || ""
+    install_dir = @install_dir
+    unless @install_root.nil? or @install_root.empty?
+      install_dir = File.join(@install_root, install_dir)
+    end
+    @cache_dir = options[:cache_dir] || install_dir
 
     @errors = []
   end
@@ -118,6 +127,78 @@ class Gem::DependencyInstaller
     @domain == :both || @domain == :remote
   end
 
+  ##
+  # Returns a list of pairs of gemspecs and source_uris that match
+  # Gem::Dependency +dep+ from both local (Dir.pwd) and remote (Gem.sources)
+  # sources.  Gems are sorted with newer gems preferred over older gems, and
+  # local gems preferred over remote gems.
+
+  def find_gems_with_sources(dep, best_only=false) # :nodoc:
+    set = Gem::AvailableSet.new
+
+    if consider_local?
+      sl = Gem::Source::Local.new
+
+      if spec = sl.find_gem(dep.name)
+        if dep.matches_spec? spec
+          set.add spec, sl
+        end
+      end
+    end
+
+    if consider_remote?
+      begin
+        # This is pulled from #spec_for_dependency to allow
+        # us to filter tuples before fetching specs.
+        tuples, errors = Gem::SpecFetcher.fetcher.search_for_dependency dep
+
+        if best_only && !tuples.empty?
+          tuples.sort! do |a,b|
+            if b[0].version == a[0].version
+              if b[0].platform != Gem::Platform::RUBY
+                1
+              else
+                -1
+              end
+            else
+              b[0].version <=> a[0].version
+            end
+          end
+          tuples = [tuples.first]
+        end
+
+        specs = []
+        tuples.each do |tup, source|
+          spec = source.fetch_spec(tup)
+        rescue Gem::RemoteFetcher::FetchError => e
+          errors << Gem::SourceFetchProblem.new(source, e)
+        else
+          specs << [spec, source]
+        end
+
+        if @errors
+          @errors += errors
+        else
+          @errors = errors
+        end
+
+        set << specs
+      rescue Gem::RemoteFetcher::FetchError => e
+        # FIX if there is a problem talking to the network, we either need to always tell
+        # the user (no really_verbose) or fail hard, not silently tell them that we just
+        # couldn't find their requested gem.
+        verbose do
+          "Error fetching remote data:\t\t#{e.message}\n" \
+            "Falling back to local-only install"
+        end
+        @domain = :local
+      end
+    end
+
+    set
+  end
+  rubygems_deprecate :find_gems_with_sources
+
   def in_background(what) # :nodoc:
     fork_happened = false
     if @build_docs_in_background && Process.respond_to?(:fork)
@@ -155,7 +236,6 @@ class Gem::DependencyInstaller
     options = {
       bin_dir: @bin_dir,
       build_args: @build_args,
-      build_jobs: @build_jobs,
       document: @document,
       env_shebang: @env_shebang,
       force: @force,
@@ -166,6 +246,7 @@ class Gem::DependencyInstaller
       user_install: @user_install,
       wrappers: @wrappers,
       build_root: @build_root,
+      install_as_default: @install_as_default,
       dir_mode: @dir_mode,
       data_mode: @data_mode,
       prog_mode: @prog_mode,
